{  1998 Arjen Wiersma - Digital Dreams Software}

unit TProgress;

interface

uses
  Windows, Classes, Controls, Forms, ComCtrls, StdCtrls, ExtCtrls, Gauges;

type
   TFProgress = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Panel4: TPanel;
    prog: TGauge;
      procedure FormClose(Sender: TObject; var Action: TCloseAction);
   private
      procedure CreateParams(var Params: TCreateParams); override;
   end;

   TWindowPositionEvent = procedure(var LeftPos, TopPos: LongInt) of object;

   TDDSProgress = class(TComponent)
   private
      FShowing: boolean;
      FPosition: integer;
      FMax: integer;
      FMin: integer;
      FCaption: string;
      FFile : string;
      FAction : string;
      FProPanel : boolean;
      FWidth: integer;
      FOnWindowPosition: TWindowPositionEvent;
      procedure SetPosition(const NewPos: integer);
      procedure SetCaption(const NewCaption: string);
      procedure SetFile(const NewFile: string);
      procedure SetPanel4(const TeZien: boolean);
      procedure SetAction(const NewAction: string);
      function ReadHeight: integer;
      function CalculateCenterPos: TPoint;
   protected

   public
      constructor Create(AOwner: TComponent); override;
      procedure Show;
      procedure Hide;
      property Position: integer write SetPosition default 0;
      property Height: integer read ReadHeight;

   published
      property Min: integer read FMin write FMin default 0;
      property Max: integer read FMax write FMax default 100;
      property Caption: string read FCaption write SetCaption;
      property Filename: string read FFile write SetFile;
      property Progress: boolean read FProPanel write SetPanel4;
      property Action: string read FAction write SetAction;
      property Width: integer read FWidth write FWidth;
      property OnPositionWindow: TWindowPositionEvent read FOnWindowPosition write FOnWindowPosition;

   end;

var
   FProgress: TFProgress;
   FParentWnd: integer;

const
   BORDER_OFFSET = 20;
   DEFAULT_WIDTH = 256;

procedure Register;

implementation

{$R *.DFM}

procedure TFProgress.CreateParams(var Params: TCreateParams);
begin
   inherited;
   with Params do begin
      WndParent := FParentWnd;
      Style := WS_DLGFRAME or WS_CAPTION;
      ExStyle := WS_EX_TOOLWINDOW;
   end;
end;

procedure TFProgress.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
   Action := caNone;
end;

constructor TDDSProgress.Create(AOwner: TComponent);
begin
   inherited;
   FMin := 0;
   FMax := 100;
   FShowing := false;
   FParentWnd := TWinControl(AOwner).Handle;
   FCaption := 'Progress';
   FWidth := DEFAULT_WIDTH;
end;

procedure TDDSProgress.Show;
var
   WindowPos: TPoint;
begin
   if FShowing then Exit;
   FProgress := TFProgress.Create(Application);
   with FProgress do begin
      Width := FWidth;
      Panel4.Visible := FProPanel;
      if panel4.Visible = False then
         ClientHeight := Panel1.height - Panel4.Height
      else
          ClientHeight := Panel1.height;
      Prog.MinValue := FMin;
      Prog.MaxValue := FMax;
      Caption := FCaption;
      WindowPos := CalculateCenterPos;
      if Assigned(FOnWindowPosition) then
         FOnWindowPosition(WindowPos.X, WindowPos.Y);
      Left := WindowPos.X;
      Top := WindowPos.Y;
      Show;
   end;
   EnableWindow(FParentWnd, false);
   FShowing := true;
end;

procedure TDDSProgress.Hide;
begin
   if not FShowing then Exit;
   EnableWindow(FParentWnd, true);
   FProgress.Free;
   FShowing := false;
end;

function TDDSProgress.CalculateCenterPos: TPoint;
var
   OwnerForm: TWinControl;
begin
   OwnerForm := Owner as TWinControl;
   Result.X := OwnerForm.Left + OwnerForm.Width div 2 - FProgress.Width div 2;
   Result.Y := OwnerForm.Top + OwnerForm.Height div 2 - FProgress.Height div 2;
end;

procedure TDDSProgress.SetPosition(const NewPos: integer);
begin
   if FShowing then begin
      FProgress.Prog.Progress := NewPos;
      Application.ProcessMessages;
   end;
end;

function TDDSProgress.ReadHeight: integer;
begin
   if FProgress <> nil then
      Result := FProgress.Height
   else
      Result := 0;
end;

procedure TDDSProgress.SetCaption(const NewCaption: string);
begin
   if FShowing then
      begin
         FProgress.Caption := NewCaption;
         Application.ProcessMessages;
      end
   else
      FCaption := NewCaption;
end;

procedure TDDSProgress.SetFile(const NewFile: string);
begin
   if FShowing then
      begin
         FProgress.Panel2.Caption := NewFile;
         Application.ProcessMessages;
      end
   else
      FFile := NewFile;
end;

procedure TDDSProgress.SetPanel4(const TeZien: Boolean);
begin
   if FShowing then
      begin
         FProgress.Panel4.Visible := TeZien;
         Application.ProcessMessages;
      end
   else
      FProPanel := TeZien;
end;

procedure TDDSProgress.SetAction(const NewAction: string);
begin
   if FShowing then
      begin
         FProgress.Panel3.Caption := NewAction;
         Application.ProcessMessages;
      end
   else
      FAction := NewAction;
end;

procedure Register;
begin
   RegisterComponents('DDS', [TDDSProgress]);
end;

end.
